///**
// * Copyright (C) 2011 - 101loops.com <dev@101loops.com>
// *
// * Licensed under the Apache License, Version 2.0 (the "License");
// * you may not use this file except in compliance with the License.
// * You may obtain a copy of the License at
// *
// *         http://www.apache.org/licenses/LICENSE-2.0
// *
// * Unless required by applicable law or agreed to in writing, software
// * distributed under the License is distributed on an "AS IS" BASIS,
// * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// * See the License for the specific language governing permissions and
// * limitations under the License.
// */
//package com.crashnote.servlet;
//
//import com.crashnote.logger.helper.AutoLogConnector;
//import com.crashnote.servlet.config.ServletConfigFactory;
//import com.crashnote.servlet.model.CustomServletResponse;
//import com.crashnote.servlet.report.ServletReporter;
//
//import javax.servlet.*;
//import javax.servlet.http.HttpServletResponse;
//import java.io.IOException;
//
///**
// * Java Servlet Filter that reports all exceptions for a HTTP request to the crashnote.com service:
// * It wraps the filter chain in a try/catch block, when an error occurred a detailed
// * report is prepared and sent.
// * <p/>
// * For web apps a simple logging appender is not sufficient since it cannot capture crucial
// * HTTP data, only a servlet filter can do this. On initialization this filter hooks into the
// * logging facilities in order to detect every kind of exception, even the ones that are already
// * handled by the according servlet.
// * <p/>
// * The filter itself is very lightweight, it only really operates when an error was detected. All
// * other requests should be hardly effected by it.
// */
//public class CrashnoteFilter2
//    implements Filter {
//
//    private ServletReporter reporter;
//    private AutoLogConnector connector;
//
//    // INTERFACE ==================================================================================
//
//    public final void init(final FilterConfig filterConfig) throws ServletException {
//
//        // at first, parse filter configuration
//        final com.crashnote.servlet.config.ServletConfig config = getConfig(filterConfig);
//
//        // if the configuration enables the filter...
//        if (config.isEnabled()) {
//
//            // ... based on config, create the central reporter service
//            reporter = config.getReporter();
//            reporter.start();
//
//            // ... finally install the appender(s)
//            connector = config.getLogConnector(reporter);
//            connector.start();
//        }
//    }
//
//    /**
//     * Every request the web application receives must go through this method first. In case the
//     * filter is disabled, the request is simply handed down the filter chain; if not, the
//     * request is wrapped in a before/after corset and exceptions are handled.
//     * <p/>
//     * Since this method is called from multiple threads concurrently, it it and must be thread-safe.
//     */
//    public final void doFilter(final ServletRequest request, final ServletResponse response,
//                               final FilterChain chain) throws IOException, ServletException {
//
//        if (reporter == null || ignoreRequest(request)) {
//
//            // filter disabled? simply hand request down the chain
//            chain.doFilter(request, response);
//
//        } else {
//
//            final CustomServletResponse customResponse =
//                new CustomServletResponse((HttpServletResponse) response);
//
//            try {
//                // initialize reporting with current request
//                reporter.beforeRequest(request);
//
//                // proceed with filter chain...
//                chain.doFilter(request, customResponse);
//
//            } catch (Throwable e) {
//                // an exception occurred! -> report it
//                reporter.uncaughtException(request, Thread.currentThread(), e);
//
//                // handle the error somehow
//                dealWithException(e);
//
//            } finally {
//                // cleanup after request
//                reporter.afterRequest(request, customResponse);
//            }
//        }
//    }
//
//    public final void destroy() {
//
//        // disconnect the appenders
//        if (connector != null) connector.stop();
//
//        // close the reporter (and let it quickly flush cached data)
//        if (reporter != null) reporter.stop();
//    }
//
//    // SHARED =====================================================================================
//
//    /**
//     * Method to decide whether to handle an incoming request or to just hand it down the filter
//     * chain. This allows the user to optionally skip certain requests.
//     *
//     * @param request the HTTP request
//     * @return true to skip reporting of errors for the request
//     */
//    protected boolean ignoreRequest(final ServletRequest request) {
//        return false;
//    }
//
//    /**
//     * Method called when an error occurred and after it was reported to crashnote.
//     * By default it re-throws the exception to let the container deal with it.
//     *
//     * @param e the error that occurred within the request chain
//     * @throws java.io.IOException
//     * @throws javax.servlet.ServletException
//     */
//    protected void dealWithException(final Throwable e) throws IOException, ServletException {
//
//        if (e instanceof IOException)
//            throw (IOException) e;
//        else if (e instanceof ServletException)
//            throw (ServletException) e;
//        else if (e instanceof RuntimeException)
//            throw (RuntimeException) e;
//        else
//            throw new RuntimeException(e);
//    }
//
//    /**
//     * Creates an instance of the internal servlet config by using the {@link javax.servlet.FilterConfig} as input.
//     */
//    protected com.crashnote.servlet.config.ServletConfig getConfig(final FilterConfig filterConfig) {
//        return new ServletConfigFactory<com.crashnote.servlet.config.ServletConfig>(filterConfig).get();
//    }
//
//}